#!/usr/bin/env python
#
# This script can be used to test the Population class with various
# optimization test functions.
#
# Author: Joshua A Haas
# Data: 2014-12-01

import math,time
import numpy as np
from population import Population

VALID = (['ackley','sphere','rosenbrock','beale','goldstein',
          'booth','bukin6','matyas','levi13','threehump',
          'easom','crosstray','eggholder','holdertable','mccormick',
          'schaffer2','schaffer4','styblinski','simionescu'])

FUNCTION = 'simionescu'

def main():
  """run the algorithm on the selected FUNCTION"""
  
  # get the parameters for the chosen FUNCTION
  (fitness,cons,mins,maxs,globmin) = getfunc(FUNCTION)
  
  # setup the Population
  p = Population(fitness,cons,mins,maxs,pop_size=1000)
  p.randpop()
  
  # keep track of iterations and time the algorithm
  iters = 0
  t1 = time.time()
  
  # run for up to 10000 generations, or until convergence
  # print current best every 10 generations
  for i in xrange(0,int(1e3)):
    add = p.evolve(maxiter=10,tol=1e-12)
    iters += add
    m = p.getmin()
    x1 = m[0][0]
    x2 = m[0][1]
    y = m[1]
    print str(iters).zfill(5)+': ['+str(x1)+','+str(x2)+'] = '+str(y)
    if add<10:
      break
  
  # print final results
  print '\n'+str(iters)+' generations in '+str(time.time()-t1)+' secs'
  print '\nCalced: ['+str(round(x1,6))+','+str(round(x2,6))+'] = '+str(round(y,6))
  x1 = globmin[0][0]
  x2 = globmin[0][1]
  y = globmin[1]
  print 'Actual: ['+str(round(x1,6))+','+str(round(x2,6))+'] = '+str(round(y,6))

def getfunc(fun):
  """return the cost function, constraint function, mins, maxs, and
  globalmin information for the given string fun or raise an error
  if the given string is invalid"""
  
  if fun=='ackley':
    fun = ackley
    cons = nocons
    mins = np.array([-5,-5])
    maxs = np.array([5,5])
    globmin = (np.array([0,0]),0)
  elif fun=='sphere':
    fun = sphere
    cons = nocons
    mins = np.array([-1000000,-1000000])
    maxs = np.array([1000000,1000000])
    globmin = (np.array([0,0]),0)
  elif fun=='rosenbrock':
    fun = rosenbrock
    cons = nocons
    mins = np.array([-1000000,-1000000])
    maxs = np.array([1000000,1000000])
    globmin = (np.array([1,1]),0)
  elif fun=='beale':
    fun = beale
    cons = nocons
    mins = np.array([-4.5,-4.5])
    maxs = np.array([4.5,4.5])
    globmin = (np.array([3,0.5]),0)
  elif fun=='goldstein':
    fun = goldstein
    cons = nocons
    mins = np.array([-2,-2])
    maxs = np.array([2,2])
    globmin = (np.array([0,-1]),3)
  elif fun=='booth':
    fun = booth
    cons = nocons
    mins = np.array([-10,-10])
    maxs = np.array([10,10])
    globmin = (np.array([1,3]),0)
  elif fun=='bukin6':
    fun = bukin6
    cons = nocons
    mins = np.array([-15,-3])
    maxs = np.array([-5,3])
    globmin = (np.array([-10,1]),0)
  elif fun=='matyas':
    fun = matyas
    cons = nocons
    mins = np.array([-10,-10])
    maxs = np.array([10,10])
    globmin = (np.array([0,0]),0)
  elif fun=='levi13':
    fun = levi13
    cons = nocons
    mins = np.array([-10,-10])
    maxs = np.array([10,10])
    globmin = (np.array([1,1]),0)
  elif fun=='threehump':
    fun = threehump
    cons = nocons
    mins = np.array([-5,-5])
    maxs = np.array([5,5])
    globmin = (np.array([0,0]),0)
  elif fun=='easom':
    fun = easom
    cons = nocons
    mins = np.array([-100,-100])
    maxs = np.array([100,100])
    globmin = (np.array([math.pi,math.pi]),-1)
  elif fun=='crosstray':
    fun = crosstray
    cons = nocons
    mins = np.array([-10,-10])
    maxs = np.array([10,10])
    globmin = (np.array([1.34941,1.34941]),-2.06261)
  elif fun=='eggholder':
    fun = eggholder
    cons = nocons
    mins = np.array([-512,-512])
    maxs = np.array([512,512])
    globmin = (np.array([512,404.2319]),-959.6407)
  elif fun=='holdertable':
    fun = holdertable
    cons = nocons
    mins = np.array([-10,-10])
    maxs = np.array([10,10])
    globmin = (np.array([8.05502,9.66459]),-19.2085)
  elif fun=='mccormick':
    fun = mccormick
    cons = nocons
    mins = np.array([-1.5,-3])
    maxs = np.array([4,4])
    globmin = (np.array([-0.54719,-1.54719]),-1.9133)
  elif fun=='schaffer2':
    fun = schaffer2
    cons = nocons
    mins = np.array([-100,-100])
    maxs = np.array([100,100])
    globmin = (np.array([0,0]),0)
  elif fun=='schaffer4':
    fun = schaffer4
    cons = nocons
    mins = np.array([-100,-100])
    maxs = np.array([100,100])
    globmin = (np.array([0,1.25313]),0.292579)
  elif fun=='styblinski':
    fun = styblinski
    cons = nocons
    mins = np.array([-5,-5])
    maxs = np.array([5,5])
    globmin = (np.array([-2.903534,-2.903534]),-78.33198)
  elif fun=='simionescu':
    fun = simionescu
    cons = simioinescucons
    mins = np.array([-1.25,-1.25])
    maxs = np.array([1.25,1.25])
    globmin = (np.array([0.84852813,-0.84852813]),-0.072)
  else:
    raise ValueError('Function "'+str(fun)+'" is not a valid function')
  return (fun,cons,mins,maxs,globmin)

# Minimum at f(0,0)=0 in x1:[-5,5] x2:[-5,5]
# Converged to f(0,0)=0 in [93,93,91,94,91]
def ackley(genes):
  """cost function for the ackley function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (-20*math.exp(-0.2*pow(0.5*(x1**2+x2**2),0.5))-
    math.exp(0.5*(math.cos(2*math.pi*x1)+math.cos(2*math.pi*x2)))+
    math.e+20)

# Minimum at f(0,0)=0 in x1:[-inf,inf] x2:[-inf,inf]
# Converged to f(0,0)=0 in [134,133,132,133,132]
def sphere(genes):
  """cost function for the sphere function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return x1**2+x2**2

# Minimum at f(1,1)=0 in x1:[-inf,inf] x2:[-inf,inf]
# Found f(1,1)=0 in 10000
# Found f(0.999323,0.998644)=0 in 10000
# Found f(1,1)=0 in 10000
# Found f(0.999996,0.999993)=0 in 10000
# Found f(6.138286,37.680212)=26.402259 in 10000
def rosenbrock(genes):
  """cost function for the rosenbrock function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (100*pow(x2-x1**2,2)+pow(x1-1,2))

# Minimum at f(3,0.5)=0 in x1:[-4.5,4.5] x2:[-4.5,4.5]
# Converged to f(3,0.5)=0 in [249,159,159,156,152]
def beale(genes):
  """cost function for the beale function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (pow(1.5-x1+x1*x2,2)+
    pow(2.25-x1+x1*pow(x2,2),2)+
    pow(2.625-x1+x1*pow(x2,3),2))

# Minimum at f(0,-1)=3 in x1:[-2,2] x2:[-2,2]
# Found f(0,-1)=3 in 10000
# Found f(0,-1)=3 in 10000
# Found f(0,-1)=3 in 10000
# Found f(0,-1)=3 in 10000
# Found f(0,-1)=3 in 10000
def goldstein(genes):
  """cost function for the goldstein function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return ((1+pow(x1+x2+1,2)*(19-14*x1+3*x1**2-14*x2+6*x1*x2+3*x2**2))*
    (30+pow(2*x1-3*x2,2)*(18-32*x1+12*x1**2+48*x2-36*x1*x2+27*x2**2)))

# Minimum at f(1,3)=0 in x1:[-10,10] x2:[-10,10]
# Converged to f(1,3)=0 in [111,111,109,112,108]
def booth(genes):
  """cost function for the booth function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (pow(x1+2*x2-7,2)+pow(2*x1+x2-5,2))

# Minimum at f(-10,1)=0 in x1:[-15,-5] x2:[-3,3]
# Converged to f(-8.619393,0.742939)=0.013806 in 444
# Converged to f(-9.822878,0.964889)=0.001771 in 503
# Converged to f(-8.992009,0.808562)=0.01008 in 512
# Converged to f(-9.617125,0.924891)=0.003829 in 492
# Converged to f(-9.065902,0.821906)=0.009341 in 451
def bukin6(genes):
  """cost function for the bukin6 function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (100*pow(abs(x2-0.01*x1**2),0.5)+0.01*abs(x1+10))

# Minimum at f(0,0)=0 in x1:[-10,10] x2:[-10,10]
# Converged to f(0,0)=0 in [144,129,135,133,135]
def matyas(genes):
  """cost function for the matyas function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (0.26*(x1**2+x2**2)-0.48*x1*x2)

# Minimum at f(1,1)=0 in x1:[-10,10] x2:[-10,10]
# Converged to f(1,1)=0 in [99,101,99,98,98]
def levi13(genes):
  """cost function for the levi13 function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (math.sin(3*math.pi*x1)**2+((x1-1)**2)*
    (1+math.sin(3*math.pi*x2)**2)+((x2-1)**2)*
    (1+math.sin(2*math.pi*x2)**2))

# Minimum at f(0,0)=0 in x1:[-5,5] x2:[-5,5]
# Converged to f(0,0)=0 in [95,97,94,97,95]
def threehump(genes):
  """cost function for the threehump function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (2*x1**2-1.05*x1**4+(x1**6)/6+x1*x2+x2**2)

# Minimum at f(pi,pi)=-1 in x1:[-100,100] x2:[-100,100]
# Found f(pi,pi)=-1 in 10000
# Found f(pi,pi)=-1 in 10000
# Found f(pi,pi)=-1 in 10000
# Found f(pi,pi)=-1 in 10000
# Found f(pi,pi)=-1 in 10000
def easom(genes):
  """cost function for the easom function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (-math.cos(x1)*math.cos(x2)*
    math.exp(-((x1-math.pi)**2+(x2-math.pi)**2)))

# Minimum at f(+-1.34941,+-1.34941) in x1:[-10,10] x2:[-10,10]
# Found f(-1.349407,1.349407)=-2.062612 in 10000
# Found f(1.349407,1.349407)=-2.062612 in 10000
# Found f(-1.349407,1.349407)=-2.062612 in 10000
# Found f(1.349407,-1.349407)=-2.062612 in 10000
# Found f(1.349407,-1.349407)=-2.062612 in 10000
# Note that this function has 4 global minima
def crosstray(genes):
  """cost function for the crosstray function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (-0.0001*(abs(math.sin(x1)*math.sin(x2)*
    math.exp(abs(100-math.sqrt(x1**2+x2**2)/math.pi)))+1)**0.1)

# Minimum at f(512,404.2319)=-959.6407 in x1:[-512,512] x2:[-512,512]
# Found f(512,404.231805)=-959.6407 in 10000
# Found f(512,404.231805)=-959.6407 in 10000
# Found f(512,404.231805)=-959.6407 in 10000
# Found f(512,404.231805)=-959.6407 in 10000
# Found f(512,404.231805)=-959.6407
def eggholder(genes):
  """cost function for the eggholder function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (-(x2+47)*math.sin(math.sqrt(abs(x2+x1/2+47)))-
    x1*math.sin(math.sqrt(abs(x1-(x2+47)))))

# Minimum at f(+-8.05502,+-9.66459)=-19.2085 in x1:[-10,10] x2:[-10,10]
# Found f(-8.05502,9.66459)=-19.208503 in 10000
# Found f(8.05502,9.66459)=-19.208503 in 10000
# Found f(-8.05502,-9.66459)=-19.208503 in 10000
# Found f(-8.05502,9.66459)=-19.208503 in 10000
# Found f(8.055023,9.66459)=-19.208503 in 10000
# Note that this function has 4 global minima
def holdertable(genes):
  """cost function for the holdertable function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (-abs(math.sin(x1)*math.cos(x2)*
    math.exp(abs(1-math.sqrt(x1**2+x2**2)/math.pi))))

# Minimum at f(-0.54719,-1.54719)=-1.9133 in x1:[-1.5,4] x2:[-3,4]
# Found f(-0.547198,-1.547198)=-1.913223 in 10000
# Found f(-0.547198,-1.547198)=-1.913223 in 10000
# Found f(-0.547198,-1.547198)=-1.913223 in 10000
# Found f(-0.547198,-1.547198)=-1.913223 in 10000
# Found f(-0.547198,-1.547198)=-1.913223 in 10000
def mccormick(genes):
  """cost function for the mccormick function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (math.sin(x1+x2)+(x1-x2)**2-1.5*x1+2.5*x2+1)

# Minimum at f(0,0)=0 in x1:[-100,100] x2:[-100,100]
# Found f(0,0)=0 in 10000
# Found f(0,0)=0 in 10000
# Found f(0,0)=0 in 10000
# Found f(0,0)=0 in 10000
# Found f(0,0)=0 in 10000
def schaffer2(genes):
  """cost function for the schaffer2 function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (0.5+((math.sin(x1**2-x2**2)**2-0.5)/
    (1+0.001*(x1**2+x2**2))**2))

# Minimum at f(0,1.25313)=0.292579 in x1:[-100,100] x2:[-100,100]
# Found f(99.991395,-99.999249)=0.500091 in 10000
# Found f(99.992107,99.999962)= 0.500091 in 10000
# Found f(-99.999465,-99.991611)=0.500091 in 10000
# Found f(99.999622,99.991767)=0.500091 in 100000
# Found f(-99.999274,99.99142)=0.500091 in 10000
def schaffer4(genes):
  """cost function for the schaffer4 function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (0.5+((math.cos(math.sin(abs(x1**2-x2**2)))-0.5)/
    (1+0.001*(x1**2+x2**2))**2))

# Minimum at f(-2.903534,-2.903534)=-78.33198 in x1:[-5,5] x2:[-5,5]
# Found f(-2.903534,-2.903534)=-78.332331 in 10000
# Found f(-2.903534,-2.903534)=-78.332331 in 10000
# Found f(-2.903534,-2.903534)=-78.332331 in 10000
# Found f(-2.903534,-2.903534)=-78.332331 in 10000
# Found f(-2.903534,-2.903534)=-78.332331 in 10000
def styblinski(genes):
  """cost function for the styblinski function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (((x1**4-16*x1**2+5*x1)+(x2**4-16*x2**2+5*x2))/2)

# Minimum at f(+-0.84852813,-+0.84852813)=-0.072 in x1:[-1.25,1.25] x2:[-1.25,1.25]
# Found f(
def simionescu(genes):
  """cost function for the simionescu function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return (0.1*x1*x2)

def simioinescucons(genes):
  """constraint function for the simionescu function"""
  
  x1 = genes[0]
  x2 = genes[1]
  return x1**2+x2**2<=(1+0.2*math.cos(8*math.atan(x1/x2)))**2

def nocons(genes):
  """constraint function for functions with only mins and maxs"""
  
  return True

if __name__ == '__main__':
  main()
